// $Id: PacketCapture.java,v 1.8 2001/06/04 00:56:20 pcharles Exp $

/***************************************************************************
 * Copyright (C) 2001, Patrick Charles and Jonas Lehmann                   *
 * Distributed under the Mozilla Public License                            *
 *   http://www.mozilla.org/NPL/MPL-1.1.txt                                *
 ***************************************************************************/
package net.sourceforge.jpcap.capture;

import net.sourceforge.jpcap.util.Timeval;
import net.sourceforge.jpcap.net.Packet;
import net.sourceforge.jpcap.net.RawPacket;
import net.sourceforge.jpcap.net.PacketFactory;


/**
 * This class is the core of packet capture in jpcap. It provides a 
 * high-level interface for capturing network packets by encapsulating 
 * libpcap.
 * <p>
 * If you want to capture network packets, implement packet listener 
 * and subscribe to an instance of this class. When packets arrive, 
 * the object will call you back via packetArrived().
 * <p>
 * Examples can be found in net.sourceforge.jpcap.tutorial.
 *
 * @author Patrick Charles and Jonas Lehmann
 * @version $Revision: 1.8 $
 * @lastModifiedBy $Author: pcharles $
 * @lastModifiedAt $Date: 2001/06/04 00:56:20 $
 */
public class PacketCapture extends PacketDispatcher implements PacketHandler
{
  /**
   * Name of the java-enabled wrapper around libpcap.
   */
  protected static String LIB_PCAP_WRAPPER = "jpcap";

  /**
   * Snapshot length. Maximum number of bytes per packet to capture.
   * For IPv4, 96 bytes guarantees that at least the headers of most 
   * packet types and protocols will get captured.
   * For IPv6, 68 is a better value?
   */
  public static int DEFAULT_SNAPLEN = 96;

  /**
   * Default capture timeout in milliseconds.
   */
  public static int DEFAULT_TIMEOUT = 1000;


  // native methods for setting up and capturing packets

  /**
   * Open a network device for data capture. Throws an exception if 
   * the device name specified is invalid. Uses default values for 
   * the capture timeout and snaplen.
   *
   * @param device the name of the network device. 
   * Examples of valid network devices on linux are 'eth0' and 'ppp0'.
   * @param promiscuous whether or not the device should be opened in 
   * promiscuous mode.
   * @return a string describing the network device. if no device can be 
   * found, null is returned.
   */
  public void open(String device, boolean promiscuous)
   throws CaptureDeviceOpenException {
    open(device, DEFAULT_SNAPLEN, promiscuous, DEFAULT_TIMEOUT);
  }

  /**
   * Open a network device for data capture.
   * 
   * @param device the name of the network device. 
   * Examples of valid network devices on linux are 'eth0' and 'ppp0'.
   * @param snaplen the 'snapshot' length. Defines the maximum number of 
   * bytes to save from each captured packet.
   * @param promiscuous whether or not the device should be opened in 
   * promiscuous mode.
   * @param timeout the packet capture timeout in milliseconds.
   * @return a string describing the network device. if no device can be 
   * found, null is returned.
   */
  public native void open(String device, int snaplen, 
                          boolean promiscuous, int timeout) 
    throws CaptureDeviceOpenException;

  /**
   * Create, compile and activate a filter from a filter expression.
   *
   * @param filterExpression the filter expression. For example, 
   * the expression "host techno" would filter only packets sent or 
   * arriving at the host named techno.
   */
  public native void setFilter(String filterExpression, boolean optimize)
    throws InvalidFilterException;

  /**
   * Capture packets.
   *
   * @param count the number of packets to capture.
   * If count is negative, capture will block forever, unless an exception 
   * is thrown.
   */
  public native void capture(int count)
    throws CapturePacketException;

  /**
   * Use the packet capture library to setup the statistics counters in 
   * this object.
   */
  private native void setupStatistics();

  /**
   * Fetch statistics on captured packets.
   * This method should not be called unless capture() was previously called.
   *
   * @return packet capture statistics.
   */
  public CaptureStatistics getStatistics() {
    setupStatistics();

    // transfer the values setup by the native method into
    // the container and return to the caller
    return new CaptureStatistics(receivedCount, droppedCount);
  }

  /**
   * Close the capture device.
   */
  public native void close();


  // static native methods to fetch capture device and network information

  /**
   * Detect a network device suitable for packet capture.
   *
   * @return a string describing the network device. if no device can be 
   * found, null is returned.
   */
  public static native String findDevice() 
    throws CaptureDeviceNotFoundException;

  /**
   * Fetch the network number for the specified device.
   *
   * @param device the name of the network device. 
   * @return the network address
   */
  public static native int getNetwork(String device)
    throws CaptureConfigurationException;

  /**
   * Fetch the network mask for the specified device.
   *
   * @param device the name of the network device. 
   * @return the netmask address
   */
  public static native int getNetmask(String device)
    throws CaptureConfigurationException;

  /**
   * Fetch the link layer type for the specified device.
   *
   * @return the link layer type code.
   */
  public static native int getLinkLayerType()
    throws CaptureConfigurationException;


  // static initialization

  static {
    System.err.print("PacketCapture: loading native library jpcap.. ");
    System.loadLibrary(LIB_PCAP_WRAPPER);
    System.err.println("ok");
  }


  // packet handler implementation

  /**
   * Handle packets arriving from the libpcap interface.
   * <p>
   * Arriving packets are dispatched to registered packet listeners.
   * If caplen is smaller than length, then the packet was truncated 
   * because the amount of data on the wire exceeded the snapshot length
   * specified when open() was called.
   *
   * @param length the length of the packet off of the wire.
   * @param caplen the number of bytes actually captured. 
   * @param seconds the seconds component of the timestamp.
   * @param useconds the microseconds component of the timestamp.
   * @param data the contents of the captured packet.
   */
  public void handlePacket(int length, int caplen, 
                           int seconds, int useconds, byte [] data) {
    // create a raw packet and abstract packet from the raw data
    RawPacket rp = new RawPacket(new Timeval(seconds, useconds),
                                 data, 
                                 length > caplen ? length - caplen : 0);
    Packet p = PacketFactory.dataToPacket(linkType, data);

    // dispatch the raw packet to registered listeners..
    dispatchRawPacket(rp);

    // dispatch the packet to listeners interested in packet objects..
    dispatchPacket(p);
  }


  /**
   * The link-type code for the currently open network device.
   * The type is queried and set in open() and then utilized when 
   * packets are received by handlePacket().
   */
  private int linkType;

  // the following are redundant with what's stored in CaptureStatistics.
  // this is temporary storage enabling the native code's statistics function
  // to manipulate the results directly in this class rather than trying 
  // to modify fields inside an object held by this object.
  // figure out how to do this more elegantly.
  private int receivedCount = 0;
  private int droppedCount = 0;

  private String _rcsid = 
    "$Id: PacketCapture.java,v 1.8 2001/06/04 00:56:20 pcharles Exp $";
}
